@@ -332,6 +332,7 @@ solaris_cdrom_eject (GnomeCDRom *cdrom,
 
 	if (gnome_cdrom_get_status (cdrom, &status, error) == FALSE) {
 		solaris_cdrom_close (lcd);
+		g_free (status);
 		return FALSE;
 	}
 
@@ -498,6 +499,7 @@ solaris_cdrom_play (GnomeCDRom *cdrom,
 
 	if (gnome_cdrom_get_status (cdrom, &status, error) == FALSE) {
 		solaris_cdrom_close (lcd);
+		g_free (status);
 		return FALSE;
 	}
 
@@ -525,6 +527,7 @@ solaris_cdrom_play (GnomeCDRom *cdrom,
 	/* Get the status again: It might have changed */
 	if (gnome_cdrom_get_status (GNOME_CDROM (lcd), &status, error) == FALSE) {
 		solaris_cdrom_close (lcd);
+		g_free (status);
 		return FALSE;
 	}
 	if (status->cd != GNOME_CDROM_STATUS_OK) {
@@ -547,6 +550,7 @@ solaris_cdrom_play (GnomeCDRom *cdrom,
 			solaris_cdrom_close (lcd);
 			return FALSE;
 		}
+		break;
 
 	case GNOME_CDROM_AUDIO_NOTHING:
 	case GNOME_CDROM_AUDIO_COMPLETE:
@@ -626,6 +630,7 @@ solaris_cdrom_pause (GnomeCDRom *cdrom,
 
 	if (gnome_cdrom_get_status (cdrom, &status, error) == FALSE) {
 		solaris_cdrom_close (lcd);
+		g_free (status);
 		return FALSE;
 	}
 
@@ -694,6 +699,7 @@ solaris_cdrom_stop (GnomeCDRom *cdrom,
 
 	if (gnome_cdrom_get_status (cdrom, &status, error) == FALSE) {
 		solaris_cdrom_close (lcd);
+		g_free (status);
 		return FALSE;
 	}
 
@@ -868,7 +874,6 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 
 	if (solaris_cdrom_open (lcd, error) == FALSE) {
 		static gboolean function_called = FALSE;
-		g_free (realstatus);
 		solaris_cdrom_close (lcd);
 		if (!function_called)
 		find_cdrom (); 
@@ -930,6 +935,7 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 
 		solaris_cdrom_close (lcd);
 		g_free (realstatus);
+		*status = NULL;
 		return FALSE;
 	}
 #else
@@ -937,7 +943,14 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 	tocentry.cdte_format = CDROM_MSF;
 	if (ioctl (cdrom->fd, CDROMREADTOCENTRY, &tocentry) < 0) {
 		g_warning ("Error getting leadout");
+		if (error)
+			*error = g_error_new (GNOME_CDROM_ERROR,
+					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
+					      "(solaris_cdrom_get_status): CDROMREADTOCENTRY ioctl failed");
+
 		solaris_cdrom_invalidate (lcd);
+		g_free (realstatus);
+		*status = NULL;
 		return FALSE;
 	}
 	if (tocentry.cdte_ctrl == CDROM_DATA_TRACK)
@@ -957,6 +970,7 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 
 		solaris_cdrom_close (lcd);
 		g_free (realstatus);
+		*status = NULL;
 		return FALSE;
 	}
 	/* get initial volume */
@@ -1192,7 +1206,6 @@ solaris_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 
 	for (i = priv->track0 - 1; i < priv->track1; i++) {
 		(*data)->offsets[i] = msf_to_frames (&priv->track_info[i].address);
-		g_print ("%d: %u\n", i, msf_to_frames (&priv->track_info[i].address));
 	}
 
 	solaris_cdrom_close (lcd);
@@ -1262,6 +1275,20 @@ gnome_cdrom_new (const char      *cdrom_device,
 		 GnomeCDRomUpdate update,
 		 GError         **error)
 {
+	int i=-1;
+	if (!solaris_cdrom_is_cdrom_device (NULL, cdrom_device, NULL)) {
+		sscanf (cdrom_device, "/vol/dev/aliases/cdrom%d", &i);
+		if (i < 0) {
+			if (error != NULL) {
+				*error = g_error_new (
+					GNOME_CDROM_ERROR,
+					GNOME_CDROM_ERROR_NOT_OPENED,
+					("%s does not appear to point to a valid CD device.\n"),cdrom_device);
+			}
+			return NULL;
+		}
+	}
+
 	return gnome_cdrom_construct (
 		g_object_new (solaris_cdrom_get_type (), NULL),
 		cdrom_device, update, GNOME_CDROM_DEVICE_TRANSIENT, error);
